/***********************************************************************
 *
 ***********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <stdarg.h>
#include <stddef.h>

#include <unistd.h>
#include <pthread.h>
#include <signal.h>
#include <sys/times.h>
#include <arpa/inet.h>

#include <epicsExport.h>
#include <devSup.h>
#include <aiRecord.h>
#include <dbAccessDefs.h>
#include <recGbl.h>
#include <errlog.h>
#include <cantProceed.h>

#include "caio.h"

static long init_record(aiRecord *);
static long read_float(aiRecord *);

struct devAiCPI
{
  long          number;
  DEVSUPFUN     report;
  DEVSUPFUN     init;
  DEVSUPFUN     init_record;
  DEVSUPFUN     get_ioint_info;
  DEVSUPFUN     read_ai;
  DEVSUPFUN     special_linconv;
} devAiCPI =
  {
    6,
    NULL,
    NULL,
    init_record,
    NULL,
    read_float,
    NULL
  };
epicsExportAddress(dset, devAiCPI);

typedef struct __cpi_ai {
  char devName[16];
  int  ch;
  int id;
} CPI_AI;


/*********************************************************************
 * single end only
 * INP, "@DEVNAME CH"
 *********************************************************************/
static long init_record(aiRecord *pAi)
{
  struct link *plink = &pAi->inp;
  int size;
  char *buf;
  char devName[16];
  int  ch;
  long rtn = -1;

  if (plink->type != INST_IO) {
    recGblRecordError(S_db_badField,(void *)pAi,
                      "devAiCPI(init_record) Illegal INP field");
    pAi->pact = 1;
    return(S_db_badField);
  }
  
  size = strlen(plink->value.instio.string) + 1;
  buf = (char *) callocMustSucceed(size, sizeof(char), "calloc failed");
  strncpy(buf, plink->value.instio.string, size);
  buf[size - 1] = '\0';

  if (sscanf(buf, "%s %d", devName, &ch) < 2) {
    errlogPrintf("devAiCPI(init_record): can't get device info for %s\n", pAi->name);
    pAi->pact = 1;
    return rtn;
  }

  // ch番号チェック(single end only)
  short id;
  long r = AioInit(devName, &id);
  if(r != 0) {
    errlogPrintf("devAiCPI(init_record): can't open device %s err=%ld\n", devName, r);
    pAi->pact = 1;
    return rtn;
  }
  short chNum;
  r = AioGetAiMaxChannels(id, &chNum);
  if(r != 0) {
    errlogPrintf("devAiCPI(init_record): AioGetAiMaxChannels name=%s id=%d err=%ld\n", devName, id, r);
    pAi->pact = 1;
    return rtn;
  }
  if((ch < 0) || (ch > chNum)) {
    errlogPrintf("devAiCPI(init_record): channel number fail ch=%d name=%s id=%d err=%ld\n", ch, devName, id, r);
    pAi->pact = 1;
    return rtn;
  }

  // single end 設定
  r = AioSetAiInputMethod(id, 0);

  // 電圧レンジ設定
  r = AioSetAiRange(id, ch, PM10);
  if(r != 0) {
    errlogPrintf("devAiCPI(read_float): can't set voltage range name=%s ch=%d err=%ld\n", devName, ch, r);
    pAi->pact = 1;
    return rtn;
  }

  CPI_AI *dpvt = (CPI_AI *)callocMustSucceed(1,
					     sizeof(CPI_AI),
					     "calloc failed");
  strcpy(dpvt->devName, devName);
  dpvt->ch = ch;
  dpvt->id = id;
  
  pAi->dpvt = dpvt;
  
  return 0;
}

/*********************************************************************
 *********************************************************************/
static long read_float(aiRecord *pAi)
{
  long rtn = -1;
  CPI_AI dev;

  memcpy(&dev, pAi->dpvt, sizeof(CPI_AI));

  short id = dev.id;

  // 電圧測定
  long data;
  long r = AioSingleAi(id, dev.ch, &data);
  if(r != 0) {
    errlogPrintf("devAiCPI(read_float): can't get voltage name=%s ch=%d err=%ld\n", dev.devName, dev.ch, r);
    pAi->pact = 1;
    return rtn;
  }
  
  pAi->rval = data;
  pAi->udf = FALSE;
  
  return 0;
}



