/***********************************************************************
 *
 ***********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <stdarg.h>
#include <stddef.h>

#include <unistd.h>
#include <pthread.h>
#include <signal.h>
#include <sys/times.h>
#include <arpa/inet.h>

#include <epicsExport.h>
#include <devSup.h>
#include <aoRecord.h>
#include <dbAccessDefs.h>
#include <recGbl.h>
#include <errlog.h>
#include <cantProceed.h>

#include "caio.h"

static long init_record(aoRecord *);
static long write_float(aoRecord *);

struct devAoCPI
{
  long          number;
  DEVSUPFUN     report;
  DEVSUPFUN     init;
  DEVSUPFUN     init_record;
  DEVSUPFUN     get_ioint_info;
  DEVSUPFUN     write_ao;
  DEVSUPFUN     special_linconv;
} devAoCPI =
  {
    6,
    NULL,
    NULL,
    init_record,
    NULL,
    write_float,
    NULL
  };
epicsExportAddress(dset, devAoCPI);

typedef struct __cpi_ao {
  char devName[16];
  int  ch;
  int  id;
} CPI_AO;


/*********************************************************************
 * single end only
 * OUT, "@DEVNAME CH"
 *********************************************************************/
static long init_record(aoRecord *pAo)
{
  struct link *plink = &pAo->out;
  int size;
  char *buf;
  char devName[16];
  int  ch;
  long rtn = -1;

  if (plink->type != INST_IO) {
    recGblRecordError(S_db_badField,(void *)pAo,
                      "devAoCPI(init_record) Illegal INP field");
    pAo->pact = 1;
    return(S_db_badField);
  }
  
  size = strlen(plink->value.instio.string) + 1;
  buf = (char *) callocMustSucceed(size, sizeof(char), "calloc faoled");
  strncpy(buf, plink->value.instio.string, size);
  buf[size - 1] = '\0';

  if (sscanf(buf, "%s %d", devName, &ch) < 2) {
    errlogPrintf("devAoCPI(init_record): can't get device info for %s\n", pAo->name);
    pAo->pact = 1;
    return rtn;
  }

  // ch番号チェック(single end only)
  short id;
  long r = AioInit(devName, &id);
  if(r != 0) {
    errlogPrintf("devAoCPI(init_record): can't open device %s err=%ld\n", devName, r);
    pAo->pact = 1;
    return rtn;
  }
  short chNum;
  r = AioGetAoMaxChannels(id, &chNum);
  if(r != 0) {
    errlogPrintf("devAoCPI(init_record): AioGetAoMaxChannels name=%s id=%d err=%ld\n", devName, id, r);
    pAo->pact = 1;
    return rtn;
  }
  if((ch < 0) || (ch > chNum)) {
    errlogPrintf("devAoCPI(init_record): channel number fail ch=%d name=%s id=%d err=%ld\n", ch, devName, id, r);
    pAo->pact = 1;
    return rtn;
  }

  // 電圧レンジ設定
  r = AioSetAoRange(id, ch, PM10);
  if(r != 0) {
    errlogPrintf("devAoCPI(write_float): can't set voltage range name=%s ch=%d val=%d err=%ld\n", devName, ch, PM10, r);
    pAo->pact = 1;
    return rtn;
  }

  CPI_AO *dpvt = (CPI_AO *)callocMustSucceed(1,
					     sizeof(CPI_AO),
					     "calloc failed");
  strcpy(dpvt->devName, devName);
  dpvt->ch = ch;
  dpvt->id = id;
  
  pAo->dpvt = dpvt;
  
  return 0;
}

/*********************************************************************
 *********************************************************************/
static long write_float(aoRecord *pAo)
{
  long rtn = -1;
  CPI_AO dev;

  memcpy(&dev, pAo->dpvt, sizeof(CPI_AO));

  short id = dev.id;

  // 電圧設定
  long data = pAo->rval;
  long r = AioSingleAo(id, dev.ch, data);
  if(r != 0) {
    errlogPrintf("devAoCPI(write_float): can't get voltage name=%s ch=%d err=%ld\n", dev.devName, dev.ch, r);
    return rtn;
  }
  
  pAo->udf = FALSE;
  
  return 0;
}



