/***********************************************************************
 *
 ***********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <stdarg.h>
#include <stddef.h>

#include <unistd.h>
#include <pthread.h>
#include <signal.h>
#include <sys/times.h>
#include <arpa/inet.h>

#include <epicsExport.h>
#include <devSup.h>
#include <biRecord.h>
#include <dbAccessDefs.h>
#include <recGbl.h>
#include <errlog.h>
#include <cantProceed.h>
#include <epicsMutex.h>

#include "cdio.h"

static long init_record(biRecord *);
static long read_bool(biRecord *);

struct devBiCPI
{
  long          number;
  DEVSUPFUN     report;
  DEVSUPFUN     init;
  DEVSUPFUN     init_record;
  DEVSUPFUN     get_ioint_info;
  DEVSUPFUN     read_bi;
  DEVSUPFUN     special_linconv;
} devBiCPI =
  {
    6,
    NULL,
    NULL,
    init_record,
    NULL,
    read_bool,
    NULL
  };
epicsExportAddress(dset, devBiCPI);

typedef struct __cpi_bi {
  char devName[16];
  int  port;
  int  ch;
  int  id;
} CPI_BI;

extern epicsMutex lock;

/*********************************************************************
 * IN, "@DEVNAME PORT CH"
 *********************************************************************/
static long init_record(biRecord *pBi)
{
  struct link *plink = &pBi->inp;
  int size;
  char *buf;
  char devName[16];
  int  ch;
  int  port;
  long rtn = -1;

  if (plink->type != INST_IO) {
    recGblRecordError(S_db_badField,(void *)pBi,
                      "devBiCPI(init_record) Illegal INP field");
    pBi->pact = 1;
    return(S_db_badField);
  }
  
  size = strlen(plink->value.instio.string) + 1;
  buf = (char *) callocMustSucceed(size, sizeof(char), "calloc faoled");
  strncpy(buf, plink->value.instio.string, size);
  buf[size - 1] = '\0';

  if (sscanf(buf, "%s %d %d", devName, &port, &ch) < 3) {
    errlogPrintf("devBiCPI(init_record): can't get device info for %s\n", pBi->name);
    pBi->pact = 1;
    return rtn;
  }

  // ch番号チェック(single end only)
  short id;
  long r = DioInit(devName, &id);
  if(r != 0) {
    errlogPrintf("devBiCPI(init_record): can't open device %s err=%ld\n", devName, r);
    pBi->pact = 1;
    return rtn;
  }
  short inPortNum, outPortNum;
  r = DioGetMaxPorts(id, &inPortNum, &outPortNum);
  if(r != 0) {
    errlogPrintf("devBiCPI(init_record): DioGetMaxCountChannels name=%s id=%d err=%ld\n", devName, id, r);
    pBi->pact = 1;
    return rtn;
  }
  if((port < 0) || (port > inPortNum)) {
    errlogPrintf("devBiCPI(init_record): port number fail ch=%d name=%s id=%d err=%ld\n", port, devName, id, r);
    pBi->pact = 1;
    return rtn;
  }
  
  CPI_BI *dpvt = (CPI_BI *)callocMustSucceed(1,
					     sizeof(CPI_BI),
					     "calloc failed");
  
  strcpy(dpvt->devName, devName);
  dpvt->port = port;
  dpvt->ch   = ch;
  dpvt->id   = id;
  
  pBi->dpvt = dpvt;


  return 0;
}

/*********************************************************************
 *********************************************************************/
static long read_bool(biRecord *pBi)
{
  long rtn = -1;
  CPI_BI dev;

  {
    epicsMutex::guard_t G(lock);

    memcpy(&dev, pBi->dpvt, sizeof(CPI_BI));
    
    short id = dev.id;

    // ビット設定
    unsigned char data;
    long r = DioInpBit(id, dev.ch, &data);
    if(r != 0) {
      errlogPrintf("devBiCPI(write_biol): can't get value name=%s ch=%d err=%ld\n", dev.devName, dev.ch, r);
      return rtn;
    }
    
    pBi->udf = FALSE;
    pBi->rval = data;
  }
  return 0;
}



