/***********************************************************************
 *
 ***********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <stdarg.h>
#include <stddef.h>

#include <unistd.h>
#include <pthread.h>
#include <signal.h>
#include <sys/times.h>
#include <arpa/inet.h>

#include <epicsExport.h>
#include <devSup.h>
#include <boRecord.h>
#include <dbAccessDefs.h>
#include <recGbl.h>
#include <errlog.h>
#include <cantProceed.h>
#include <epicsMutex.h>

#include "cdio.h"

static long init_record(boRecord *);
static long write_bool(boRecord *);

struct devBoCPI
{
  long          number;
  DEVSUPFUN     report;
  DEVSUPFUN     init;
  DEVSUPFUN     init_record;
  DEVSUPFUN     get_ioint_info;
  DEVSUPFUN     write_bo;
  DEVSUPFUN     special_linconv;
} devBoCPI =
  {
    6,
    NULL,
    NULL,
    init_record,
    NULL,
    write_bool,
    NULL
  };
epicsExportAddress(dset, devBoCPI);

typedef struct __cpi_bo {
  char devName[16];
  int  port;
  int  ch;
  int  id;
} CPI_BO;

epicsMutex lock;

/*********************************************************************
 * OUT, "@DEVNAME PORT CH"
 *********************************************************************/
static long init_record(boRecord *pBo)
{
  struct link *plink = &pBo->out;
  int size;
  char *buf;
  char devName[16];
  int  ch;
  int  port;
  long rtn = -1;

  if (plink->type != INST_IO) {
    recGblRecordError(S_db_badField,(void *)pBo,
                      "devBoCPI(init_record) Illegal INP field");
    pBo->pact = 1;
    return(S_db_badField);
  }
  
  size = strlen(plink->value.instio.string) + 1;
  buf = (char *) callocMustSucceed(size, sizeof(char), "calloc faoled");
  strncpy(buf, plink->value.instio.string, size);
  buf[size - 1] = '\0';

  if (sscanf(buf, "%s %d %d", devName, &port, &ch) < 3) {
    errlogPrintf("devBoCPI(init_record): can't get device info for %s\n", pBo->name);
    pBo->pact = 1;
    return rtn;
  }

  // ch番号チェック(single end only)
  short id;
  long r = DioInit(devName, &id);
  if(r != 0) {
    errlogPrintf("devBoCPI(init_record): can't open device %s err=%ld\n", devName, r);
    pBo->pact = 1;
    return rtn;
  }
  short inPortNum, outPortNum;
  r = DioGetMaxPorts(id, &inPortNum, &outPortNum);
  if(r != 0) {
    errlogPrintf("devBoCPI(init_record): DioGetMaxCountChannels name=%s id=%d err=%ld\n", devName, id, r);
    pBo->pact = 1;
    return rtn;
  }
  if((port < 0) || (port > outPortNum)) {
    errlogPrintf("devBoCPI(init_record): port number fail port=%d name=%s id=%d err=%ld\n", port, devName, id, r);
    pBo->pact = 1;
    return rtn;
  }
  
  CPI_BO *dpvt = (CPI_BO *)callocMustSucceed(1,
					     sizeof(CPI_BO),
					     "calloc failed");
  strcpy(dpvt->devName, devName);
  dpvt->ch = ch;
  dpvt->id = id;
  pBo->dpvt = dpvt;
  
  return 0;
}

/*********************************************************************
 *********************************************************************/
static long write_bool(boRecord *pBo)
{
  long rtn = -1;
  CPI_BO dev;

  {
    epicsMutex::guard_t G(lock);
    
    memcpy(&dev, pBo->dpvt, sizeof(CPI_BO));

    short id = dev.id;
    
    // ビット設定
    unsigned char data = pBo->rval;
    long r = DioOutBit(id, dev.ch, data);
    if(r != 0) {
      errlogPrintf("devBoCPI(write_bool): can't get voltage name=%s ch=%d err=%ld\n", dev.devName, dev.ch, r);
      return rtn;
    }
  
    pBo->udf = FALSE;
  }
  return 0;
}



