/***********************************************************************
 *
 ***********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <stdarg.h>
#include <stddef.h>

#include <unistd.h>
#include <pthread.h>
#include <signal.h>
#include <sys/times.h>
#include <arpa/inet.h>

#include <epicsExport.h>
#include <devSup.h>
#include <mbbiDirectRecord.h>
#include <dbAccessDefs.h>
#include <recGbl.h>
#include <errlog.h>
#include <cantProceed.h>
#include <epicsMutex.h>

#include "cdio.h"

static long init_record(mbbiDirectRecord *);
static long read_mbbiDirect(mbbiDirectRecord *);

struct devMbbiDirectCPI
{
  long          number;
  DEVSUPFUN     report;
  DEVSUPFUN     init;
  DEVSUPFUN     init_record;
  DEVSUPFUN     get_ioint_info;
  DEVSUPFUN     read_mbbiDirect;
  DEVSUPFUN     special_linconv;
} devMbbiDirectCPI =
  {
    6,
    NULL,
    NULL,
    init_record,
    NULL,
    read_mbbiDirect,
    NULL
  };
epicsExportAddress(dset, devMbbiDirectCPI);

typedef struct __cpi_mbbiDirect {
  char devName[16];
  int  ch;
  int  id;
} CPI_MBBIDIRECT;

extern epicsMutex lock;

/*********************************************************************
 * IN, "@DEVNAME CH"
 *********************************************************************/
static long init_record(mbbiDirectRecord *pMbbiDirect)
{
  struct link *plink = &pMbbiDirect->inp;
  int size;
  char *buf;
  char devName[16];
  int  ch;
  long rtn = -1;

  if (plink->type != INST_IO) {
    recGblRecordError(S_db_badField,(void *)pMbbiDirect,
                      "devMbbiDirectCPI(init_record) Illegal INP field");
    pMbbiDirect->pact = 1;
    return(S_db_badField);
  }
  
  size = strlen(plink->value.instio.string) + 1;
  buf = (char *) callocMustSucceed(size, sizeof(char), "calloc faoled");
  strncpy(buf, plink->value.instio.string, size);
  buf[size - 1] = '\0';

  if (sscanf(buf, "%s %d", devName, &ch) < 2) {
    errlogPrintf("devMbbiDirectCPI(init_record): can't get device info for %s\n", pMbbiDirect->name);
    pMbbiDirect->pact = 1;
    return rtn;
  }

  // ch番号チェック(single end only)
  short id;
  long r = DioInit(devName, &id);
  if(r != 0) {
    errlogPrintf("devMbbiDirectCPI(init_record): can't open device %s err=%ld\n", devName, r);
    pMbbiDirect->pact = 1;
    return rtn;
  }
  short inPortNum, outPortNum;
  r = DioGetMaxPorts(id, &inPortNum, &outPortNum);
  if(r != 0) {
    errlogPrintf("devMbbiDirectCPI(init_record): DioGetMaxCountChannels name=%s id=%d err=%ld\n", devName, id, r);
    pMbbiDirect->pact = 1;
    return rtn;
  }
  if((ch < 0) || (ch > inPortNum)) {
    errlogPrintf("devMbbiDirectCPI(init_record): channel number fail ch=%d name=%s id=%d err=%ld\n", ch, devName, id, r);
    pMbbiDirect->pact = 1;
    return rtn;
  }
  
  CPI_MBBIDIRECT *dpvt = (CPI_MBBIDIRECT *)callocMustSucceed(1,
					     sizeof(CPI_MBBIDIRECT),
					     "calloc failed");
  strcpy(dpvt->devName, devName);
  dpvt->ch = ch;
  dpvt->id = id;
  
  pMbbiDirect->dpvt = dpvt;
  
  return 0;
}

/*********************************************************************
 *********************************************************************/
static long read_mbbiDirect(mbbiDirectRecord *pMbbiDirect)
{
  long rtn = -1;
  CPI_MBBIDIRECT dev;

  {
    epicsMutex::guard_t G(lock);
    memcpy(&dev, pMbbiDirect->dpvt, sizeof(CPI_MBBIDIRECT));

    short id = dev.id;
    
    // ビット設定
    unsigned char data;
    long r = DioInpByte(id, dev.ch, &data);
    if(r != 0) {
      errlogPrintf("devMbbiDirectCPI(write_mbbiDirect): can't get value name=%s ch=%d err=%ld\n", dev.devName, dev.ch, r);
      pMbbiDirect->pact = 1;
      return rtn;
    }

    pMbbiDirect->rval = data;
    pMbbiDirect->udf = FALSE;
  }
  
  return 0;
}



